@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-kajongg is free software you can redistribute it and/or modify
+Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation either version 2 of the License, or
 (at your option) any later version.
@@ -18,52 +18,60 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-import csv, resource, random
+import random
 
 from twisted.spread import pb
 from twisted.python.failure import Failure
 from twisted.internet.defer import Deferred, succeed, DeferredList
-from PyQt4.QtCore import Qt, QTimer
-from PyQt4.QtGui import QDialog, QVBoxLayout, QGridLayout, \
-    QLabel, QPushButton, \
+from qt import Qt, QTimer
+from qt import QDialog, QVBoxLayout, QGridLayout, \
+    QLabel, QPushButton, QWidget, \
     QProgressBar, QRadioButton, QSpacerItem, QSizePolicy
 
-from kde import Sorry, Information, QuestionYesNo, KIcon, \
-    DialogIgnoringEscape
-
-from util import m18n, logWarning, logException, \
-    logInfo, logDebug
+from kde import usingKDE, KIcon, KDialog
+from dialogs import Sorry, Information, QuestionYesNo, KDialogIgnoringEscape
+from guiutil import decorateWindow
+from log import m18n, logWarning, logException, logDebug
 from message import Message, ChatMessage
 from chat import ChatWindow
-from common import Options, SingleshotOptions, Internal, Preferences, Debug, isAlive
+from common import Options, SingleshotOptions, Internal, Debug, isAlive
+from common import nativeString, nativeStringArgs
 from query import Query
 from board import Board
 from client import Client, ClientTable
 from tables import TableList, SelectRuleset
 from sound import Voice
-import intelligence
-import altint
 from login import Connection
 from rule import Ruleset
 from game import PlayingGame
 from visible import VisiblePlayingGame
 
-class SelectChow(DialogIgnoringEscape):
+
+class SelectChow(KDialogIgnoringEscape):
+
     """asks which of the possible chows is wanted"""
+
     def __init__(self, chows, propose, deferred):
-        DialogIgnoringEscape.__init__(self)
-        self.setWindowTitle('Kajongg')
+        KDialogIgnoringEscape.__init__(self)
+        decorateWindow(self)
+        self.setButtons(KDialog.NoButton)
         self.chows = chows
         self.selectedChow = None
         self.deferred = deferred
-        layout = QVBoxLayout(self)
-        layout.addWidget(QLabel(m18n('Which chow do you want to expose?')))
+        layout = QVBoxLayout()
+        label = QLabel(m18n('Which chow do you want to expose?'))
+        layout.addWidget(label)
+        layout.setAlignment(label, Qt.AlignHCenter)
         self.buttons = []
         for chow in chows:
-            button = QRadioButton('-'.join([chow[0][1], chow[1][1], chow[2][1]]), self)
+            button = QRadioButton('{}-{}-{}'.format(*(x.value for x in chow)))
             self.buttons.append(button)
             layout.addWidget(button)
+            layout.setAlignment(button, Qt.AlignHCenter)
             button.toggled.connect(self.toggled)
+        widget = QWidget(self)
+        widget.setLayout(layout)
+        self.setMainWidget(widget)
         for idx, chow in enumerate(chows):
             if chow == propose:
                 self.buttons[idx].setFocus()
@@ -76,29 +84,31 @@ class SelectChow(DialogIgnoringEscape):
             self.accept()
             self.deferred.callback((Message.Chow, self.selectedChow))
 
-    def closeEvent(self, event):
-        """allow close only if a chow has been selected"""
-        if self.selectedChow:
-            event.accept()
-        else:
-            event.ignore()
 
-class SelectKong(DialogIgnoringEscape):
+class SelectKong(KDialogIgnoringEscape):
+
     """asks which of the possible kongs is wanted"""
+
     def __init__(self, kongs, deferred):
-        DialogIgnoringEscape.__init__(self)
-        self.setWindowTitle('Kajongg')
+        KDialogIgnoringEscape.__init__(self)
+        decorateWindow(self)
+        self.setButtons(0)
         self.kongs = kongs
         self.selectedKong = None
         self.deferred = deferred
-        layout = QVBoxLayout(self)
-        layout.addWidget(QLabel(m18n('Which kong do you want to declare?')))
+        layout = QVBoxLayout()
+        label = QLabel(m18n('Which kong do you want to declare?'))
+        layout.addWidget(label)
+        layout.setAlignment(label, Qt.AlignHCenter)
         self.buttons = []
         for kong in kongs:
             button = QRadioButton((kong[0].name()), self)
             self.buttons.append(button)
             layout.addWidget(button)
             button.toggled.connect(self.toggled)
+        widget = QWidget(self)
+        widget.setLayout(layout)
+        self.setMainWidget(widget)
 
     def toggled(self, dummyChecked):
         """a radiobutton has been toggled"""
@@ -108,24 +118,21 @@ class SelectKong(DialogIgnoringEscape):
             self.accept()
             self.deferred.callback((Message.Kong, self.selectedKong))
 
-    def closeEvent(self, event):
-        """allow close only if a chow has been selected"""
-        if self.selectedKong:
-            event.accept()
-        else:
-            event.ignore()
 
 class DlgButton(QPushButton):
+
     """special button for ClientDialog"""
+
     def __init__(self, message, parent):
         QPushButton.__init__(self, parent)
         self.message = message
         self.client = parent.client
         self.setText(message.buttonCaption())
 
-    def decorate(self, tile):
+    def decorate(self, uiTile):
         """give me caption, shortcut, tooltip, icon"""
-        txt, warn, _ = self.message.toolTip(self, tile.tile if tile else None)
+        txt, warn, _ = self.message.toolTip(
+            self, uiTile.tile if uiTile else None)
         if not txt:
             txt = self.message.i18nName  # .replace(i18nShortcut, '&'+i18nShortcut, 1)
         self.setToolTip(txt)
@@ -149,11 +156,14 @@ class DlgButton(QPushButton):
         else:
             self.setIcon(KIcon())
 
+
 class ClientDialog(QDialog):
+
     """a simple popup dialog for asking the player what he wants to do"""
+
     def __init__(self, client, parent=None):
         QDialog.__init__(self, parent)
-        self.setWindowTitle(m18n('Choose') + ' - Kajongg')
+        decorateWindow(self, m18n('Choose'))
         self.setObjectName('ClientDialog')
         self.client = client
         self.layout = QGridLayout(self)
@@ -185,8 +195,8 @@ class ClientDialog(QDialog):
 
     def __declareButton(self, message):
         """define a button"""
-        maySay = self.client.sayable[message]
-        if Preferences.showOnlyPossibleActions and not maySay:
+        maySay = self.client.game.myself.sayable[message]
+        if Internal.Preferences.showOnlyPossibleActions and not maySay:
             return
         btn = DlgButton(message, self)
         btn.setAutoDefault(True)
@@ -199,16 +209,17 @@ class ClientDialog(QDialog):
             return
         for button in self.buttons:
             button.decorate(self.client.game.myself.handBoard.focusTile)
-        for tile in self.client.game.myself.handBoard.lowerHalfTiles():
+        for uiTile in self.client.game.myself.handBoard.lowerHalfTiles():
             txt = []
             for button in self.buttons:
-                _, _, tileTxt = button.message.toolTip(button, tile.tile)
+                _, _, tileTxt = button.message.toolTip(button, uiTile.tile)
                 if tileTxt:
                     txt.append(tileTxt)
             txt = '<br><br>'.join(txt)
-            tile.setToolTip(txt)
+            uiTile.setToolTip(txt)
         if self.client.game.activePlayer == self.client.game.myself:
-            Internal.field.handSelectorChanged(self.client.game.myself.handBoard)
+            Internal.scene.handSelectorChanged(
+                self.client.game.myself.handBoard)
 
     def checkTiles(self):
         """does the logical state match the displayed tiles?"""
@@ -224,15 +235,15 @@ class ClientDialog(QDialog):
         focus a proposed tile depending on the action."""
         result = self.buttons[0]
         game = self.client.game
-        if game.autoPlay or Preferences.propose:
-            answer, parameter = self.client.intelligence.selectAnswer(
+        if game.autoPlay or Internal.Preferences.propose:
+            answer, parameter = game.myself.intelligence.selectAnswer(
                 self.messages())
             result = [x for x in self.buttons if x.message == answer][0]
             result.setFocus()
             if answer in [Message.Discard, Message.OriginalCall]:
-                for tileItem in game.myself.handBoard.tiles:
-                    if tileItem.tile == parameter:
-                        game.myself.handBoard.focusTile = tileItem
+                for uiTile in game.myself.handBoard.uiTiles:
+                    if uiTile.tile is parameter:
+                        game.myself.handBoard.focusTile = uiTile
         return result
 
     def askHuman(self, move, answers, deferred):
@@ -258,40 +269,55 @@ class ClientDialog(QDialog):
         if not myTurn:
             msecs = 50
             self.progressBar.setMinimum(0)
-            self.progressBar.setMaximum(game.ruleset.claimTimeout * 1000 // msecs)
+            self.progressBar.setMaximum(
+                game.ruleset.claimTimeout * 1000 // msecs)
             self.progressBar.reset()
             self.timer.start(msecs)
 
     def placeInField(self):
         """place the dialog at bottom or to the right depending on space."""
-        field = Internal.field
-        cwi = field.centralWidget()
-        view = field.centralView
+        mainWindow = Internal.scene.mainWindow
+        cwi = mainWindow.centralWidget()
+        view = mainWindow.centralView
         geometry = self.geometry()
         if not self.btnHeight:
             self.btnHeight = self.buttons[0].height()
         vertical = view.width() > view.height() * 1.2
         if vertical:
             height = (len(self.buttons) + 1) * self.btnHeight * 1.2
-            width = (cwi.width() - cwi.height() ) // 2
+            width = (cwi.width() - cwi.height()) // 2
             geometry.setX(cwi.width() - width)
-            geometry.setY(min(cwi.height()//3, cwi.height() - height))
+            geometry.setY(min(cwi.height() // 3, cwi.height() - height))
         else:
             handBoard = self.client.game.myself.handBoard
             if not handBoard:
                 # we are in the progress of logging out
                 return
-            hbLeftTop = view.mapFromScene(handBoard.mapToScene(handBoard.rect().topLeft()))
-            hbRightBottom = view.mapFromScene(handBoard.mapToScene(handBoard.rect().bottomRight()))
+            hbLeftTop = view.mapFromScene(
+                handBoard.mapToScene(handBoard.rect().topLeft()))
+            hbRightBottom = view.mapFromScene(
+                handBoard.mapToScene(handBoard.rect().bottomRight()))
             width = hbRightBottom.x() - hbLeftTop.x()
             height = self.btnHeight
             geometry.setY(cwi.height() - height)
             geometry.setX(hbLeftTop.x())
         for idx, btn in enumerate(self.buttons + [self.progressBar]):
-            self.layout.addWidget(btn, idx+1 if vertical else 0, idx+1 if not vertical else 0)
+            self.layout.addWidget(
+                btn,
+                idx +
+                1 if vertical else 0,
+                idx +
+                1 if not vertical else 0)
         idx = len(self.buttons) + 2
-        spacer = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Expanding)
-        self.layout.addItem(spacer, idx if vertical else 0, idx if not vertical else 0)
+        spacer = QSpacerItem(
+            20,
+            20,
+            QSizePolicy.Expanding,
+            QSizePolicy.Expanding)
+        self.layout.addItem(
+            spacer,
+            idx if vertical else 0,
+            idx if not vertical else 0)
 
         geometry.setWidth(width)
         geometry.setHeight(height)
@@ -305,15 +331,19 @@ class ClientDialog(QDialog):
         """the progressboard wants an update"""
         pBar = self.progressBar
         if isAlive(pBar):
-            pBar.setValue(pBar.value()+1)
+            pBar.setValue(pBar.value() + 1)
             pBar.setVisible(True)
             if pBar.value() == pBar.maximum():
-                # timeout: we always return the original default answer, not the one with focus
+                # timeout: we always return the original default answer, not
+                # the one with focus
                 self.selectButton()
                 pBar.setVisible(False)
 
     def selectButton(self, button=None):
         """select default answer. button may also be of type Message."""
+        if self.answered:
+            # sometimes we get this event twice
+            return
         self.timer.stop()
         self.answered = True
         if button is None:
@@ -323,10 +353,10 @@ class ClientDialog(QDialog):
             answer = button
         else:
             answer = button.message
-        if not self.client.sayable[answer]:
+        if not self.client.game.myself.sayable[answer]:
             Sorry(m18n('You cannot say %1', answer.i18nName))
             return
-        Internal.field.clientDialog = None
+        Internal.scene.clientDialog = None
         self.deferred.callback(answer)
 
     def selectedAnswer(self, dummyChecked):
@@ -335,29 +365,35 @@ class ClientDialog(QDialog):
         if game and not game.autoPlay:
             self.selectButton(self.sender())
 
+
 class HumanClient(Client):
+
     """a human client"""
     # pylint: disable=too-many-public-methods
     humanClients = []
+
     def __init__(self):
-        aiClass = self.__findAI([intelligence, altint], Options.AI)
-        if not aiClass:
-            raise Exception('intelligence %s is undefined' % Options.AI)
-        Client.__init__(self, intelligence=aiClass)
+        Client.__init__(self)
         HumanClient.humanClients.append(self)
         self.table = None
         self.ruleset = None
         self.beginQuestion = None
         self.tableList = TableList(self)
-        Connection(self).login().addCallbacks(self.__loggedIn, self.__loginFailed)
+        Connection(
+            self).login(
+        ).addCallbacks(
+            self.__loggedIn,
+             self.__loginFailed)
 
     @staticmethod
     def shutdownHumanClients(exception=None):
         """close connections to servers except maybe one"""
         clients = HumanClient.humanClients
+
         def done():
             """return True if clients is cleaned"""
             return len(clients) == 0 or (exception and clients == [exception])
+
         def disconnectedClient(dummyResult, client):
             """now the client is really disconnected from the server"""
             if client in clients:
@@ -373,7 +409,11 @@ class HumanClient(Client):
         deferreds = []
         for client in clients[:]:
             if client != exception and client.connection:
-                deferreds.append(client.logout().addCallback(disconnectedClient, client))
+                deferreds.append(
+                    client.logout(
+                    ).addCallback(
+                        disconnectedClient,
+                        client))
         return DeferredList(deferreds)
 
     def __loggedIn(self, connection):
@@ -383,17 +423,19 @@ class HumanClient(Client):
         self.name = connection.username
         self.tableList.show()
         voiceId = None
-        if Preferences.uploadVoice:
+        if Internal.Preferences.uploadVoice:
             voice = Voice.locate(self.name)
             if voice:
                 voiceId = voice.md5sum
             if Debug.sound and voiceId:
-                logDebug('%s sends own voice %s to server' % (self.name, voiceId))
+                logDebug(
+                    u'%s sends own voice %s to server' %
+                    (self.name, voiceId))
         maxGameId = Query('select max(id) from game').records[0][0]
         maxGameId = int(maxGameId) if maxGameId else 0
         self.callServer('setClientProperties',
-            Internal.dbIdent,
-            voiceId, maxGameId, Internal.version).addCallbacks(self.__initTableList, self.__versionError)
+                        Internal.db.identifier,
+                        voiceId, maxGameId, Internal.version).addCallbacks(self.__initTableList, self.__versionError)
 
     def __initTableList(self, dummy):
         """first load of the list. Process options like --demo, --table, --join"""
@@ -403,32 +445,30 @@ class HumanClient(Client):
             self.__requestNewTableFromServer(SingleshotOptions.table).addCallback(
                 self.__showTables).addErrback(self.tableError)
             if Debug.table:
-                logDebug('%s: --table lets us open an new table %d' % (self.name, SingleshotOptions.table))
+                logDebug(
+                    u'%s: --table lets us open an new table %d' %
+                    (self.name, SingleshotOptions.table))
             SingleshotOptions.table = False
         elif SingleshotOptions.join:
             Internal.autoPlay = False
             self.callServer('joinTable', SingleshotOptions.join).addCallback(
                 self.__showTables).addErrback(self.tableError)
             if Debug.table:
-                logDebug('%s: --join lets us join table %s' % (self.name, self._tableById(SingleshotOptions.join)))
+                logDebug(
+                    u'%s: --join lets us join table %s' %
+                    (self.name, self._tableById(SingleshotOptions.join)))
             SingleshotOptions.join = False
         elif not self.game and (Internal.autoPlay or (not self.tables and self.hasLocalServer())):
-            self.__requestNewTableFromServer().addCallback(self.__newLocalTable).addErrback(self.tableError)
+            self.__requestNewTableFromServer().addCallback(
+                self.__newLocalTable).addErrback(self.tableError)
         else:
             self.__showTables()
 
     @staticmethod
     def __loginFailed(dummy):
         """as the name says"""
-        Internal.field.startingGame = False
-
-    @staticmethod
-    def __findAI(modules, aiName):
-        """list of all alternative AIs defined in altint.py"""
-        for modul in modules:
-            for key, value in modul.__dict__.items():
-                if key == 'AI' + aiName:
-                    return value
+        if Internal.scene:
+            Internal.scene.startingGame = False
 
     def isRobotClient(self):
         """avoid using isinstance, it would import too much for kajonggserver"""
@@ -445,7 +485,7 @@ class HumanClient(Client):
 
     def hasLocalServer(self):
         """True if we are talking to a Local Game Server"""
-        return self.connection and self.connection.useSocket
+        return self.connection and self.connection.url.isLocalHost
 
     def __updateTableList(self):
         """if it exists"""
@@ -480,22 +520,32 @@ class HumanClient(Client):
             if self.hasLocalServer():
                 # when playing a local game, only show pending tables with
                 # previously selected ruleset
-                self.tables = list(x for x in self.tables if x.ruleset == self.ruleset)
+                self.tables = list(
+                    x for x in self.tables if x.ruleset == self.ruleset)
+                assert self.tables
         if len(self.tables):
             self.__updateTableList()
 
     def remote_newTables(self, tables):
         """update table list"""
         assert len(tables)
+
         def gotRulesets(result):
             """the server sent us the wanted ruleset definitions"""
             for ruleset in result:
-                Ruleset.cached(ruleset).save(copy=True) # make it known to the cache and save in db
+                Ruleset.cached(
+                    ruleset).save(
+                )  # make it known to the cache and save in db
             return tables
         rulesetHashes = set(x[1] for x in tables)
-        needRulesets = list(x for x in rulesetHashes if not Ruleset.hashIsKnown(x))
+        needRulesets = list(
+            x for x in rulesetHashes if not Ruleset.hashIsKnown(x))
         if needRulesets:
-            self.callServer('needRulesets', needRulesets).addCallback(gotRulesets).addCallback(self.__receiveTables)
+            self.callServer(
+                'needRulesets',
+                needRulesets).addCallback(
+                    gotRulesets).addCallback(
+                        self.__receiveTables)
         else:
             self.__receiveTables(tables)
 
@@ -536,25 +586,29 @@ class HumanClient(Client):
         """others chat to me"""
         chatLine = ChatMessage(data)
         if Debug.chat:
-            logDebug('got chatLine: %s' % chatLine)
+            logDebug(u'got chatLine: %s' % chatLine)
         table = self._tableById(chatLine.tableid)
         if not chatLine.isStatusMessage and not table.chatWindow:
             ChatWindow(table)
         if table.chatWindow:
             table.chatWindow.receiveLine(chatLine)
 
-    def readyForGameStart(self, tableid, gameid, wantedGame, playerNames, shouldSave=True,
+    def readyForGameStart(
+        self, tableid, gameid, wantedGame, playerNames, shouldSave=True,
             gameClass=None):
         """playerNames are in wind order ESWN"""
         if gameClass is None:
-            if Internal.field:
+            if Options.gui:
                 gameClass = VisiblePlayingGame
             else:
                 gameClass = PlayingGame
+
         def clientReady():
             """macro"""
-            return Client.readyForGameStart(self, tableid, gameid, wantedGame, playerNames,
+            return Client.readyForGameStart(
+                self, tableid, gameid, wantedGame, playerNames,
                 shouldSave, gameClass)
+
         def answered(result):
             """callback, called after the client player said yes or no"""
             self.beginQuestion = None
@@ -563,10 +617,11 @@ class HumanClient(Client):
                 return clientReady()
             else:
                 return Message.NoGameStart
+
         def cancelled(dummy):
             """the user does not want to start now. Back to table list"""
             if Debug.table:
-                logDebug('%s: Readyforgamestart returns Message.NoGameStart for table %s' % (
+                logDebug(u'%s: Readyforgamestart returns Message.NoGameStart for table %s' % (
                     self.name, self._tableById(tableid)))
             self.table = None
             self.beginQuestion = None
@@ -574,15 +629,20 @@ class HumanClient(Client):
                 self.__updateTableList()
                 self.tableList.show()
             return Message.NoGameStart
-        if sum(not x[1].startswith('Robot ') for x in playerNames) == 1:
-            # we play against 3 robots and we already told the server to start: no need to ask again
+        if sum(not x[1].startswith(u'Robot ') for x in playerNames) == 1:
+            # we play against 3 robots and we already told the server to start:
+            # no need to ask again
             return clientReady()
         assert not self.table
         assert self.tables
         self.table = self._tableById(tableid)
         if not self.table:
-            raise pb.Error('client.readyForGameStart: tableid %d unknown' % tableid)
-        msg = m18n("The game on table <numid>%1</numid> can begin. Are you ready to play now?", tableid)
+            raise pb.Error(
+                'client.readyForGameStart: tableid %d unknown' %
+                tableid)
+        msg = m18n(
+            "The game on table <numid>%1</numid> can begin. Are you ready to play now?",
+            tableid)
         self.beginQuestion = QuestionYesNo(msg, modal=False, caption=self.name).addCallback(
             answered).addErrback(cancelled)
         return self.beginQuestion
@@ -596,37 +656,40 @@ class HumanClient(Client):
         if not self.connection:
             # disconnected meanwhile
             return
-        if Internal.field:
+        if Options.gui:
             # update the balances in the status bar:
-            Internal.field.updateGUI()
+            Internal.mainWindow.updateGUI()
         assert not self.game.isFirstHand()
-        return Information(m18n("Ready for next hand?"), modal=True).addCallback(answered)
+        return Information(m18n("Ready for next hand?"), modal=usingKDE).addCallback(answered)
 
     def ask(self, move, answers):
         """server sends move. We ask the user. answers is a list with possible answers,
         the default answer being the first in the list."""
-        if not Internal.field:
+        if not Options.gui:
             return Client.ask(self, move, answers)
-        self._computeSayable(move, answers)
+        self.game.myself.computeSayable(move, answers)
         deferred = Deferred()
         deferred.addCallback(self.__askAnswered)
         deferred.addErrback(self.__answerError, move, answers)
         iAmActive = self.game.myself == self.game.activePlayer
         self.game.myself.handBoard.setEnabled(iAmActive)
-        field = Internal.field
-        oldDialog = field.clientDialog
+        scene = Internal.scene
+        oldDialog = scene.clientDialog
         if oldDialog and not oldDialog.answered:
             raise Exception('old dialog %s:%s is unanswered, new Dialog: %s/%s' % (
                 str(oldDialog.move),
-                str([x.name for x in oldDialog.buttons]),
+                str([x.message.name for x in oldDialog.buttons]),
                 str(move), str(answers)))
         if not oldDialog or not oldDialog.isVisible():
             # always build a new dialog because if we change its layout before
             # reshowing it, sometimes the old buttons are still visible in which
-            # case the next dialog will appear at a lower position than it should
-            field.clientDialog = ClientDialog(self, field.centralWidget())
-        assert field.clientDialog.client is self
-        field.clientDialog.askHuman(move, answers, deferred)
+            # case the next dialog will appear at a lower position than it
+            # should
+            scene.clientDialog = ClientDialog(
+                self,
+                scene.mainWindow.centralWidget())
+        assert scene.clientDialog.client is self
+        scene.clientDialog.askHuman(move, answers, deferred)
         return deferred
 
     def __selectChow(self, chows):
@@ -634,12 +697,13 @@ class HumanClient(Client):
         Since we might return a Deferred to be sent to the server,
         which contains Message.Chow plus selected Chow, we should
         return the same tuple here"""
+        intelligence = self.game.myself.intelligence
         if self.game.autoPlay:
-            return Message.Chow, self.intelligence.selectChow(chows)
+            return Message.Chow, intelligence.selectChow(chows)
         if len(chows) == 1:
             return Message.Chow, chows[0]
-        if Preferences.propose:
-            propose = self.intelligence.selectChow(chows)
+        if Internal.Preferences.propose:
+            propose = intelligence.selectChow(chows)
         else:
             propose = None
         deferred = Deferred()
@@ -650,7 +714,7 @@ class HumanClient(Client):
     def __selectKong(self, kongs):
         """which possible kong do we want to declare?"""
         if self.game.autoPlay:
-            return Message.Kong, self.intelligence.selectKong(kongs)
+            return Message.Kong, self.game.myself.intelligence.selectKong(kongs)
         if len(kongs) == 1:
             return Message.Kong, kongs[0]
         deferred = Deferred()
@@ -665,10 +729,11 @@ class HumanClient(Client):
         myself = self.game.myself
         if answer in [Message.Discard, Message.OriginalCall]:
             # do not remove tile from hand here, the server will tell all players
-            # including us that it has been discarded. Only then we will remove it.
+            # including us that it has been discarded. Only then we will remove
+            # it.
             myself.handBoard.setEnabled(False)
             return answer, myself.handBoard.focusTile.tile
-        args = self.sayable[answer]
+        args = self.game.myself.sayable[answer]
         assert args
         if answer == Message.Chow:
             return self.__selectChow(args)
@@ -684,10 +749,14 @@ class HumanClient(Client):
 
     def __answerError(self, answer, move, answers):
         """an error happened while determining the answer to server"""
-        logException('%s %s %s %s' % (self.game.myself.name if self.game else 'NOGAME', answer, move, answers))
+        logException(
+            '%s %s %s %s' %
+            (self.game.myself.name if self.game else 'NOGAME', answer, move, answers))
 
     def remote_abort(self, tableid, message, *args):
         """the server aborted this game"""
+        message = nativeString(message)
+        args = nativeStringArgs(args)
         if self.table and self.table.tableid == tableid:
             # translate Robot to Roboter:
             if self.game:
@@ -695,9 +764,6 @@ class HumanClient(Client):
             logWarning(m18n(message, *args))
             if self.game:
                 self.game.close()
-                if self.game.autoPlay:
-                    if Internal.field:
-                        Internal.field.close()
 
     def remote_gameOver(self, tableid, message, *args):
         """the game is over"""
@@ -705,37 +771,23 @@ class HumanClient(Client):
             """now that the user clicked the 'game over' prompt away, clean up"""
             if self.game:
                 self.game.rotateWinds()
-                if Options.csv:
-                    gameWinner = max(self.game.players, key=lambda x: x.balance)
-                    writer = csv.writer(open(Options.csv,'a'), delimiter=';')
-                    if Debug.process:
-                        self.game.csvTags.append('MEM:%s' % resource.getrusage(resource.RUSAGE_SELF).ru_maxrss)
-                    row = [self.game.ruleset.name, Options.AI, str(self.game.seed), ','.join(self.game.csvTags)]
-                    for player in sorted(self.game.players, key=lambda x: x.name):
-                        row.append(player.name.encode('utf-8'))
-                        row.append(player.balance)
-                        row.append(player.wonCount)
-                        row.append(1 if player == gameWinner else 0)
-                    writer.writerow(row)
-                    del writer
-                if self.game.autoPlay and Internal.field:
-                    Internal.field.close()
-                else:
-                    self.game.close().addCallback(Client.quitProgram)
+                self.game.close().addCallback(Internal.mainWindow.close)
         assert self.table and self.table.tableid == tableid
-        if Internal.field:
+        if Internal.scene:
             # update the balances in the status bar:
-            Internal.field.updateGUI()
-        logInfo(m18n(message, *args), showDialog=True).addCallback(yes)
+            Internal.scene.mainWindow.updateGUI()
+        Information(m18n(message, *args)).addCallback(yes)
 
     def remote_serverDisconnects(self, result=None):
         """we logged out or or lost connection to the server.
         Remove visual traces depending on that connection."""
         if Debug.connections and result:
-            logDebug('server %s disconnects: %s' % (self.connection.url, result))
+            logDebug(
+                u'server %s disconnects: %s' %
+                (self.connection.url, result))
         self.connection = None
         game = self.game
-        self.game = None # avoid races: messages might still arrive
+        self.game = None  # avoid races: messages might still arrive
         if self.tableList:
             self.tableList.hide()
             self.tableList = None
@@ -743,21 +795,25 @@ class HumanClient(Client):
             HumanClient.humanClients.remove(self)
         if self.beginQuestion:
             self.beginQuestion.cancel()
-        field = Internal.field
-        if field and game and field.game == game:
-            game.close() # TODO: maybe issue a Sorry first?
+        scene = Internal.scene
+        if scene and game and scene.game == game:
+            scene.game = None
 
     def serverDisconnected(self, dummyReference):
         """perspective calls us back"""
         if self.connection and (Debug.traffic or Debug.connections):
-            logDebug('perspective notifies disconnect: %s' % self.connection.url)
+            logDebug(
+                u'perspective notifies disconnect: %s' %
+                self.connection.url)
         self.remote_serverDisconnects()
 
     @staticmethod
     def __versionError(err):
         """log the twisted error"""
         logWarning(err.getErrorMessage())
-        Internal.field.abortGame()
+        if Internal.game:
+            Internal.game.close()
+            Internal.game = None
         return err
 
     @staticmethod
@@ -765,7 +821,7 @@ class HumanClient(Client):
         """find out which game we want to start on the table"""
         result = SingleshotOptions.game
         if not result or result == '0':
-            result = str(int(random.random() * 10**9))
+            result = str(int(random.random() * 10 ** 9))
         SingleshotOptions.game = None
         return result
 
@@ -787,8 +843,9 @@ class HumanClient(Client):
         """as the name says"""
         if ruleset is None:
             ruleset = self.ruleset
+        self.connection.ruleset = ruleset  # side effect: saves ruleset as last used for server
         return self.callServer('newTable', ruleset.hash, Options.playOpen,
-            Internal.autoPlay, self.__wantedGame(), tableid).addErrback(self.tableError)
+                               Internal.autoPlay, self.__wantedGame(), tableid).addErrback(self.tableError)
 
     def newTable(self):
         """TableList uses me as a slot"""
@@ -815,7 +872,7 @@ class HumanClient(Client):
     def logout(self, dummyResult=None):
         """clean visual traces and logout from server"""
         def loggedout(result, connection):
-            """TODO: do we need this?"""
+            """end the connection from client side"""
             connection.connector.disconnect()
             return result
         if self.connection:
@@ -825,6 +882,20 @@ class HumanClient(Client):
         else:
             return succeed(None)
 
+    def __logCallServer(self, *args):
+        """for Debug.traffic"""
+        debugArgs = list(args[:])
+        if Debug.neutral:
+            if debugArgs[0] == 'ping':
+                return
+            if debugArgs[0] == 'setClientProperties':
+                debugArgs[1] = u'DBID'
+                debugArgs[3] = u'GAMEID'
+        if self.game:
+            self.game.debug('callServer(%s)' % repr(debugArgs))
+        else:
+            logDebug(u'callServer(%s)' % repr(debugArgs))
+
     def callServer(self, *args):
         """if we are online, call server"""
         if self.connection:
@@ -832,10 +903,8 @@ class HumanClient(Client):
                 args = args[1:]
             try:
                 if Debug.traffic:
-                    if self.game:
-                        self.game.debug('callServer(%s)' % repr(args))
-                    else:
-                        logDebug('callServer(%s)' % repr(args))
+                    self.__logCallServer(*args)
+
                 def callServerError(result):
                     """if serverDisconnected has been called meanwhile, just ignore msg about
                     connection lost in a non-clean fashion"""
@@ -843,8 +912,10 @@ class HumanClient(Client):
                         return result
                 return self.connection.perspective.callRemote(*args).addErrback(callServerError)
             except pb.DeadReferenceError:
-                logWarning(m18n('The connection to the server %1 broke, please try again later.',
-                                  self.connection.url))
+                logWarning(
+                    m18n(
+                        'The connection to the server %1 broke, please try again later.',
+                          self.connection.url))
                 self.remote_serverDisconnects()
                 return succeed(None)
         else:
